#define SN "DimmableLEDx"
#define SV "1.0"

#include <MySensor.h>
#include <SPI.h>
#define FADE_DELAY 10  // Delay in ms for each percentage fade up/down (10ms = 1s full-range dim)

MySensor gw;

#include "dSensor.h"

void setup()
{
#ifdef DEBUG
  Serial.println( SN );
#endif

  gw.begin( incomingMessage );
  gw.sendSketchInfo(SN, SV);

  for (int i = 0; i < numChannels; i++) {
    // Register the LED Dimmable Light with the gateway
    gw.present( i, S_DIMMER );

    // Pull the gateway's current dim level - restore light level upon sendor node power-up
    gw.request( i, V_DIMMER );
  }
#ifdef DEBUG
  Serial.print( "channels " );
  Serial.println ( numChannels );
#endif

}

void loop()
{
  gw.process();

}

void incomingMessage(const MyMessage &message) {
  int si = message.sensor;
#ifdef DEBUG
  Serial.print( "message " );
  Serial.print( si );
  Serial.println( "" );
#endif

  if (message.type == V_LIGHT || message.type == V_DIMMER) {

    //  Retrieve the power or dim level from the incoming request message
    int requestedLevel = atoi( message.data );
    // Adjust incoming level if this is a V_LIGHT variable update [0 == off, 1 == on]
    requestedLevel *= ( message.type == V_LIGHT ? 100 : 1 );

    requestedLevel = constrain(requestedLevel, 0, 100);

    /*
      // Clip incoming level to valid range of 0 to 100
      requestedLevel = requestedLevel > 100 ? 100 : requestedLevel;
      requestedLevel = requestedLevel < 0   ? 0   : requestedLevel;
    */
#ifdef DEBUG
    Serial.print( "Changing level to " );
    Serial.print( requestedLevel );
    Serial.print( ", from " );
    Serial.println( registers[si].currLvl );
#endif

    fadeToLevel( si, requestedLevel );

    // Inform the gateway of the current DimmableLED's SwitchPower1 and LoadLevelStatus value...
    gw.send(registers[si].lightMsg.set(registers[si].currLvl > 0 ? 1 : 0));

    // hek comment: Is this really nessesary?
    gw.send( registers[si].dimmerMsg.set(registers[si].currLvl) );


  }
}

/***
    This method provides a graceful fade up/down effect
*/
void fadeToLevel( int si, int toLevel ) {

  int delta = ( toLevel - registers[si].currLvl ) < 0 ? -1 : 1;

  while ( registers[si].currLvl != toLevel ) {
    registers[si].currLvl += delta;
    analogWrite( registers[si].ledPin, (int)(registers[si].currLvl / 100. * 255) );
    delay( FADE_DELAY );
  }
}
